%--------------------------------------------------------------------------
% processEEGdata - This script processes the EEG data. 
% EEG data is plotted, lost packages is found, EEG signals are high pass 
% filtered, fourier transformed, and the mean values for power and 
% frequency in alpha, beta and theta waves are found and written to .txt 
% files. Their trends are plotted.  
%
% created: 16.02.2015 - Lise Bjoerkvoll
%--------------------------------------------------------------------------
      
for sim = 1:length(simResults)
    
    EEGdata=[];                                  % Create vectors for storing trajectories
    EEGdata= [remainingData; simResults{sim}];   % Code for appending the rest of last simulationResult               
    [EEGstart, EEGstop] = findTimeStamps(EEGdata, first, second); % Find time staps
    plotEEGdata(EEGdata(EEGstart:EEGstop, :))                         % Plot EEG signals against time
    lostPackages(EEGdata)                        % Check if there are any lost packages



%% High pass filter 

    d= highpass();                               % Create high pass filter
    filteredData(:, 1)=EEGdata(:,1); 
    for j = 2:15
        filteredData(:, j) = filter(d,EEGdata(:, j));   % Apply filter
    end   

%% Taking fourier transform and finding power and frequency mean of whole data set

    if typeOfProgram==2             % Analyse a data set
        for i=1:length(SENSOR)
            [Px, fVals]= takeFourierTransform(filteredData(EEGstart:EEGstop,SENSOR(i)));     % Take fourier transform
            [alpha_pow, beta_pow, theta_pow,  alpha_freq, beta_freq, theta_freq] = findMeanPowerAndFrequency(Px, fVals);    % Finding means for different frequency bands
            plotPSD(Px, fVals, length(EEGdata(EEGstart:EEGstop,18)))      % plot PSD and single sided amplitude spectrum for every sensor
            
            alphaPow = alphaPow + alpha_pow;
            betaPow = betaPow + beta_pow;
            alphaFreq = alphaFreq + alpha_freq;
            betaFreq = betaFreq + beta_freq;
            thetaFreq = thetaFreq + theta_freq;
            thetaPow = thetaPow + theta_pow;
        end

        alpha_freq = alphaFreq/length(SENSOR);
        beta_freq = betaFreq/length(SENSOR);
        theta_freq = thetaFreq/length(SENSOR);
        alpha_pow = alphaPow/length(SENSOR);
        beta_pow = betaPow/length(SENSOR);
        theta_pow = thetaPow/length(SENSOR);
        
      % Write average values to .txt file                
      writeToFile=[round(alpha_freq) round(alpha_pow) round(beta_freq) round(beta_pow) round(theta_freq) round(theta_pow)];
      csvwrite('C:\Users\brainSensingApplication\analyseDataSet.txt',writeToFile);
    end


 
%% Taking fourier transform and finding power and frequency mean of parts of data - for running time application
    sec =  round(EEGdata(end,18)-EEGdata(1,18));    % total seconds of extracted data
    startPlace = 1;
    previousStartPlace = 1;
    flagEndProcessing=0;

    alphaPow = 0;
    alpha_pow = 0;
    alphaFreq = 0;
    alpha_freq = 0;

    betaPow = 0;
    beta_pow = 0;
    betaFreq = 0;  
    beta_freq = 0;

    thetaPow = 0;
    theta_pow = 0;
    thetaFreq = 0;
    theta_freq = 0;

    
    if sim==1 || boolProcessingHasBegun==0
        initData= EEGdata(1,18);
    else
        initData=temp;
    end
    for i = 1:2*sec+1
        h=[];
        for j= startPlace: length(EEGdata(:,18)) 
            if EEGdata(j,18)>= initData+ i*0.5 %
                temp= initData+ i*0.5;
                boolProcessingHasBegun=1;   %data processing has started
                startPlace = j;
                for k=1:length(SENSOR)
                    h(k,:)=filteredData(previousStartPlace:startPlace, SENSOR(k));
                    t = EEGdata(previousStartPlace:startPlace,18)';
                end
                h=h';
                previousStartPlace = startPlace; 
                break
            end
            if i==2*sec+1 &&flagEndProcessing==0
                remainingData=EEGdata(previousStartPlace:length(EEGdata(:, SENSOR(1))), :); %saving the rest of the data
                flagEndProcessing=1;
            end
        end
        % Taking the fourier transform of the last 5 seconds:
        if isempty(h)==0
            counter = counter + 1;
            for sensorLength = 1: length(h(1,:))    %go trough all sensors
                newest5secOfData{sensorLength, counter} = h(:, sensorLength); % 
            end
                if counter == 10 || flagFirst5secHasPassed == 1
                   flagFirst5secHasPassed=1;
                   for sensorLength = 1: length(h(1,:))     %go trough all sensors
                       dummy = counter+1; 
                       for m = 1:10
                          if dummy == 11
                            dummy = 1;                    
                          end
                          descending5secOfData = [descending5secOfData; newest5secOfData{sensorLength, dummy}];
                          dummy = dummy + 1; 
                       end
                       
                      [Px, fVals]= takeFourierTransform(descending5secOfData);
                       [alpha_pow, beta_pow, theta_pow,  alpha_freq, beta_freq, theta_freq] = findMeanPowerAndFrequency(Px, fVals);
                       alphaPow = alphaPow + alpha_pow;
                       betaPow = betaPow + beta_pow;
                       thetaPow = thetaPow + theta_pow;
                       alphaFreq = alphaFreq + alpha_freq;
                       betaFreq = betaFreq + beta_freq;
                       thetaFreq = thetaFreq + theta_freq;
                   end
               alpha_pow = alphaPow/length(h(1,:));
               beta_pow = betaPow/length(h(1,:));
               theta_pow = thetaPow/length(h(1,:));
               alpha_freq = alphaFreq/length(h(1,:));
               beta_freq = betaFreq/length(h(1,:));
               theta_freq = thetaFreq/length(h(1,:));
               alphaPow = 0;
               betaPow = 0;
               thetaPow = 0;
               alphaFreq = 0;
               betaFreq = 0;
               thetaFreq = 0;
               descending5secOfData = [];
               timeStamp = timeStamp+0.5;
               timeSeries = [timeSeries timeStamp];
               alphaFreqDevelopment = [alphaFreqDevelopment alpha_freq]; 
               alphaPowDevelopment = [alphaPowDevelopment alpha_pow]; 
               betaFreqDevelopment = [betaFreqDevelopment beta_freq]; 
               betaPowDevelopment = [betaPowDevelopment beta_pow]; 
               thetaFreqDevelopment = [thetaFreqDevelopment theta_freq]; 
               thetaPowDevelopment = [thetaPowDevelopment theta_pow];
               
               if typeOfProgram==1      % Running time application
                   % write mean values to .txt file for use in ABBsimulator every 0.5 sec                
                   writeToSimulator=[round(alpha_freq) round(alpha_pow) round(beta_freq) round(beta_pow) round(theta_freq) round(theta_pow)];
                   csvwrite('C:\Users\brainSensingApplication\calculatedValues.txt',writeToSimulator);
               end
               if counter==10
                  counter=0;
               end
           end
        end
    end
end

% Save data for plotting
if typeOfProgram==2     % Analyse data set
    if frontal
        alphaFreqDevelopment_F = alphaFreqDevelopment;
        alphaPowDevelopment_F = alphaPowDevelopment; 
        betaFreqDevelopment_F = betaFreqDevelopment; 
        betaPowDevelopment_F = betaPowDevelopment; 
        thetaFreqDevelopment_F = thetaFreqDevelopment; 
        thetaPowDevelopment_F = thetaPowDevelopment; 
    end

    if parietal
        alphaFreqDevelopment_P = alphaFreqDevelopment;
        alphaPowDevelopment_P = alphaPowDevelopment; 
        betaFreqDevelopment_P = betaFreqDevelopment; 
        betaPowDevelopment_P = betaPowDevelopment; 
        thetaFreqDevelopment_P = thetaFreqDevelopment; 
        thetaPowDevelopment_P = thetaPowDevelopment; 
    end

    if occipital
        alphaFreqDevelopment_O = alphaFreqDevelopment;
        alphaPowDevelopment_O = alphaPowDevelopment; 
        betaFreqDevelopment_O = betaFreqDevelopment; 
        betaPowDevelopment_O = betaPowDevelopment; 
        thetaFreqDevelopment_O = thetaFreqDevelopment; 
        thetaPowDevelopment_O = thetaPowDevelopment; 
    end

    if all
        alphaFreqDevelopment_All = alphaFreqDevelopment;
        alphaPowDevelopment_All = alphaPowDevelopment; 
        betaFreqDevelopment_All = betaFreqDevelopment; 
        betaPowDevelopment_All = betaPowDevelopment; 
        thetaFreqDevelopment_All = thetaFreqDevelopment; 
        thetaPowDevelopment_All = thetaPowDevelopment; 
    end
    
    if frontal || parietal|| occipital || all
        counterPlot = counterPlot+1;
        if counterPlot==4      % When development in all brain areas is caluculated, they can be plotted 
            plotTrends(alphaFreqDevelopment_F, alphaFreqDevelopment_P, alphaFreqDevelopment_O, alphaFreqDevelopment_All, 1) 
            plotTrends(alphaPowDevelopment_F, alphaPowDevelopment_P, alphaPowDevelopment_O, alphaPowDevelopment_All, 2) 
            plotTrends(betaFreqDevelopment_F, betaFreqDevelopment_P, betaFreqDevelopment_O, betaFreqDevelopment_All, 3) 
            plotTrends(betaPowDevelopment_F, betaPowDevelopment_P, betaPowDevelopment_O, betaPowDevelopment_All, 4) 
            plotTrends(thetaFreqDevelopment_F, thetaFreqDevelopment_P, thetaFreqDevelopment_O, thetaFreqDevelopment_All, 5) 
            plotTrends(thetaPowDevelopment_F, thetaPowDevelopment_P, thetaPowDevelopment_O, thetaPowDevelopment_All, 6) 
        end
    end
end