%--------------------------------------------------------------------------
% takeFourierTransform - This function takes the fourier transform of a
% data set, and returns frequency range for PSD and the power of each
% component
% 
% created: 10.02.2015 - Lise Bjoerkvoll
%--------------------------------------------------------------------------

% Fourier Transform 
function [Px, fVals]= takeFourierTransform(h)
N = 2400;
Fs = 128;
t = (0:N-1)/Fs;

    y = h;                            % Signal
    fs =  128;                         % Sampling frequency of gathered data in Hz
    Nsamps = length(y);               % Window length
    NFFT= 2^nextpow2(Nsamps);        % 1024 - number of samples taken from time domain signal - Transform length
    fVals = fs/Nsamps*(0:Nsamps/2-1); % Frequency range for PSD, nyquist frequency of signal = F/2 = 128/2 = 64
    f = fs/2*linspace(0,1,NFFT/2+1);  % Frequency range for Amplitude Spectrum, nyquist frequency of signal = F/2 = 128/2 = 64


    % Finding Amplitude Spectrum
    Y=fft(y,NFFT)*(1/fs);


    % Power Spectral Density
    fVals = fs/NFFT*(0:NFFT/2-1); % Frequency range for PSD
    X=fft(h, NFFT);               % Fourier transform
    Px=X.*conj(X)/(NFFT);         % Power of each component 


 end


